@@ -1,8 +1,10 @@
-# LoclaeInfo.py (c) 2006 Canonical, released under the GPL
+# LocaleInfo.py (c) 2006 Canonical, released under the GPL
 #
 # a helper class to get locale info
 
-import string
+from __future__ import print_function
+from __future__ import absolute_import
+
 import re            
 import subprocess
 import gettext
@@ -11,7 +13,8 @@ import pwd
 import sys
 import dbus
 import warnings
-import macros
+
+from LanguageSelector import macros
 
 from gettext import gettext as _
 from xml.etree.ElementTree import ElementTree
@@ -19,7 +22,7 @@ from xml.etree.ElementTree import ElementTree
 class LocaleInfo(object):
     " class with handy functions to parse the locale information "
     
-    environments = ["/etc/default/locale", "/etc/environment"]
+    environments = ["/etc/default/locale"]
     def __init__(self, languagelist_file, datadir):
         self._datadir = datadir
         LANGUAGELIST = os.path.join(datadir, 'data', languagelist_file)
@@ -37,7 +40,7 @@ class LocaleInfo(object):
         
         # read lang file
         et = ElementTree(file="/usr/share/xml/iso-codes/iso_639.xml")
-        it = et.getiterator('iso_639_entry')
+        it = et.iter('iso_639_entry')
         for elm in it:
             lang = elm.attrib["name"]
             if "iso_639_1_code" in elm.attrib:
@@ -53,7 +56,7 @@ class LocaleInfo(object):
         self._lang['zh-hant'] = _("Chinese (traditional)")
         # end hack
         et = ElementTree(file="/usr/share/xml/iso-codes/iso_639_3.xml")
-        it = et.getiterator('iso_639_3_entry')
+        it = et.iter('iso_639_3_entry')
         for elm in it:
             lang = elm.attrib["name"]
             code = elm.attrib["id"]
@@ -62,7 +65,7 @@ class LocaleInfo(object):
         
         # read countries
         et = ElementTree(file="/usr/share/xml/iso-codes/iso_3166.xml")
-        it = et.getiterator('iso_3166_entry')
+        it = et.iter('iso_3166_entry')
         for elm in it:
             if "common_name" in elm.attrib:
                 descr = elm.attrib["common_name"]
@@ -75,19 +78,20 @@ class LocaleInfo(object):
             self._country[code] = descr
             
         # read the languagelist
-        for line in open(LANGUAGELIST):
-            tmp = line.strip()
-            if tmp.startswith("#") or tmp == "":
-                continue
-            w = tmp.split(";")
-            # FIXME: the latest localechoosers "languagelist" does
-            # no longer have this field for most languages, so
-            # deal with it and don't set LANGUAGE then
-            # - the interessting question is what to do
-            # if LANGUAGE is already set and the new
-            localeenv = w[6].split(":")
-            #print localeenv
-            self._languagelist[localeenv[0]] = '%s' % w[6]
+        with open(LANGUAGELIST) as f:
+            for line in f:
+                tmp = line.strip()
+                if tmp.startswith("#") or tmp == "":
+                    continue
+                w = tmp.split(";")
+                # FIXME: the latest localechoosers "languagelist" does
+                # no longer have this field for most languages, so
+                # deal with it and don't set LANGUAGE then
+                # - the interessting question is what to do
+                # if LANGUAGE is already set and the new
+                localeenv = w[6].split(":")
+                #print(localeenv)
+                self._languagelist[localeenv[0]] = '%s' % w[6]
 
     def lang(self, code):
         """ map language code to language name """
@@ -105,8 +109,9 @@ class LocaleInfo(object):
         """ return a list of locales available on the system
             (running locale -a) """
         locales = []
-        p = subprocess.Popen(["locale", "-a"], stdout=subprocess.PIPE)
-        for line in string.split(p.communicate()[0], "\n"):
+        p = subprocess.Popen(["locale", "-a"], stdout=subprocess.PIPE,
+                             universal_newlines=True)
+        for line in p.communicate()[0].split("\n"):
             tmp = line.strip()
             if tmp.find('.utf8') < 0:
                 continue
@@ -115,7 +120,7 @@ class LocaleInfo(object):
             locale = macr["LOCALE"]
             if not locale in locales:
                 locales.append(locale)
-        #print locales
+        #print(locales)
         return locales
 
     def translate_language(self, lang):
@@ -137,7 +142,7 @@ class LocaleInfo(object):
 
 #        macr = macros.LangpackMacros(self._datadir, locale)
 
-#        #(lang, country) = string.split(locale, "_")
+#        #(lang, country) = locale.split("_")
 #        country = macr['CCODE']
 #        current_language = None
 #        if "LANGUAGE" in os.environ:
@@ -170,14 +175,14 @@ class LocaleInfo(object):
         if len(macr["CCODE"]) > 0:
             country_name = self.translate_country(macr["CCODE"])
             # get all locales for this language
-            l = filter(lambda k: k.startswith(macr['LCODE']), self.generated_locales())
+            l = [k for k in self.generated_locales() if k.startswith(macr['LCODE'])]
             # only show region/country if we have more than one 
             if (allCountries == False and len(l) > 1) or allCountries == True:
                 mycountry = self.country(macr['CCODE'])
                 if mycountry:
                     returnVal = "%s (%s)" % (lang_name, country_name)
         if len(macr["VARIANT"]) > 0:
-            returnVal = "%s - %s" % (returnVal, macr["VARIANT"])
+            returnVal = "%s - %s" % (returnVal, macr["VARIANT"].encode('UTF-8'))
         
         if native == True:
             if current_language:
@@ -187,10 +192,10 @@ class LocaleInfo(object):
         return returnVal
          
 #        if "_" in locale:
-#            #(lang, country) = string.split(locale, "_")
+#            #(lang, country) = locale.split("_")
 #            (lang_name, country_name) = self.translate_locale(locale)
 #            # get all locales for this language
-#            l = filter(lambda k: k.startswith(macr['LCODE']), self.generated_locales())
+#            l = [k for k in self.generated_locales() if k.startswith(macr['LCODE'])]
 #            # only show region/country if we have more than one 
 #            if len(l) > 1:
 #                mycountry = self.country(macr['CCODE'])
@@ -207,142 +212,120 @@ class LocaleInfo(object):
             the LANGUAGE enviroment variable.
             E.g: en_DK -> en_DK:en
         """
+        if not code:
+            return ''
         macr = macros.LangpackMacros(self._datadir, code)
         langcode = macr['LCODE']
         locale = macr['LOCALE']
         # first check if we got somethign from languagelist
         if locale in self._languagelist:
-            return self._languagelist[locale]
+            langlist = self._languagelist[locale]
         # if not, fall back to "dumb" behaviour
-        if locale == langcode:
-            return locale
-        return "%s:%s" % (locale, langcode)
+        elif locale == langcode:
+            langlist = locale
+        else:
+            langlist = "%s:%s" % (locale, langcode)
+        if not (langlist.endswith(':en') or langlist == 'en'):
+            langlist = "%s:en" % langlist
+        return langlist
 
     def getUserDefaultLanguage(self):
-        """
-        Reads '~/.profile' if present; otherwise - or if '~/.profile' doesn't set any
-        values - AccountsService is queried.
-        Scans for LANG and LANGUAGE variable settings and returns a list [LANG, LANGUAGE].
-        In the case of AccountsService, we only have one language, not the full LANGUAGE
-        variable compatible string. Therefore we generate one from the provided language.
-        Likewise, if LANGUAGE is not defined, generate a string from the provided LANG value.
-        """
-        lang = ''
+        formats = ''
         language = ''
         result = []
-        fname = os.path.expanduser("~/.profile")
+        fname = os.path.expanduser("~/.pam_environment")
         if os.path.exists(fname) and \
            os.access(fname, os.R_OK):
-            for line in open(fname):
-                match_lang = re.match(r'export LANG=(.*)$',line)
-                if match_lang:
-                    lang = match_lang.group(1).strip('"')
-                match_language = re.match(r'export LANGUAGE=(.*)$',line)
-                if match_language:
-                    language = match_language.group(1).strip('"')
-        if len(language) == 0:
+            with open(fname) as f:
+                for line in f:
+                    match_language = re.match(r'LANGUAGE=(.*)$',line)
+                    if match_language:
+                        language = match_language.group(1)
+        user_name = pwd.getpwuid(os.geteuid()).pw_name
+        try:
             bus = dbus.SystemBus()
-            if 'fontconfig-voodoo' in sys.argv[0] and os.getenv('SUDO_USER'):
-                # handle 'sudo fontconfig-voodoo --auto' correctly
-                user_name = os.environ['SUDO_USER']
-            else:
-                user_name = pwd.getpwuid(os.geteuid()).pw_name
-            try:
-                obj = bus.get_object('org.freedesktop.Accounts', '/org/freedesktop/Accounts')
-                iface = dbus.Interface(obj, dbus_interface='org.freedesktop.Accounts')
-                user_path = iface.FindUserByName(user_name)
+            obj = bus.get_object('org.freedesktop.Accounts', '/org/freedesktop/Accounts')
+            iface = dbus.Interface(obj, dbus_interface='org.freedesktop.Accounts')
+            user_path = iface.FindUserByName(user_name)
 
-                obj = bus.get_object('org.freedesktop.Accounts', user_path)
-                iface = dbus.Interface(obj, dbus_interface='org.freedesktop.DBus.Properties')
+            obj = bus.get_object('org.freedesktop.Accounts', user_path)
+            iface = dbus.Interface(obj, dbus_interface='org.freedesktop.DBus.Properties')
+            formats = iface.Get('org.freedesktop.Accounts.User', 'FormatsLocale')
+            if len(language) == 0:
                 firstLanguage = iface.Get('org.freedesktop.Accounts.User', 'Language')
                 language = self.makeEnvString(firstLanguage)
-            except Exception as msg:
-                # a failure here shouldn't trigger a fatal error
-                warnings.warn(msg.args[0].encode('UTF-8'))
-                pass
-        if len(lang) == 0 and "LANG" in os.environ:
-            lang = os.environ["LANG"]
-        if len(lang) > 0:
-            if len(language) == 0:
-                if "LANGUAGE" in os.environ and os.environ["LANGUAGE"] != lang:
-                    language = os.environ["LANGUAGE"]
-                else:
-                    language = self.makeEnvString(lang)
-        result.append(lang)
+        except Exception as msg:
+            # a failure here shouldn't trigger a fatal error
+            warnings.warn(msg.args[0].encode('UTF-8'))
+            pass
+        if len(language) == 0 and "LANGUAGE" in os.environ:
+            language = os.environ["LANGUAGE"]
+        if len(formats) == 0 and "LC_NAME" in os.environ:
+            formats = os.environ["LC_NAME"]
+        if len(formats) == 0 and "LANG" in os.environ:
+            formats = os.environ["LANG"]
+        if len(formats) > 0 and len(language) == 0:
+            language = self.makeEnvString(formats)
+        result.append(formats)
         result.append(language)
         return result
 
     def getSystemDefaultLanguage(self):
-        """
-        Reads '/etc/default/locale' if present, and if not '/etc/environment'.
-        Scans for LANG and LANGUAGE variable settings and returns a list [LANG, LANGUAGE].
-        """
         lang = ''
+        formats = ''
         language = ''
         result = []
         for fname in self.environments:
             if os.path.exists(fname) and \
                os.access(fname, os.R_OK):
-                for line in open(fname):
-                    # support both LANG="foo" and LANG=foo
-                    if line.startswith("LANG"):
-                        line = line.replace('"','')
-                    match_lang = re.match(r'LANG=(.*)$',line)
-                    if match_lang:
-                        lang = match_lang.group(1).strip('"')
-                    if line.startswith("LANGUAGE"):
-                        line = line.replace('"','')
-                    match_language = re.match(r'LANGUAGE=(.*)$',line)
-                    if match_language:
-                        language = match_language.group(1).strip('"')
-                if len(lang) > 0:
-                    break
+                with open(fname) as f:
+                    for line in f:
+                        # support both LANG="foo" and LANG=foo
+                        if line.startswith("LANG"):
+                            line = line.replace('"','')
+                        match_lang = re.match(r'LANG=(.*)$',line)
+                        if match_lang:
+                            lang = match_lang.group(1)
+                        if line.startswith("LC_TIME"):
+                            line = line.replace('"','')
+                        match_formats = re.match(r'LC_TIME=(.*)$',line)
+                        if match_formats:
+                            formats = match_formats.group(1)
+                        if line.startswith("LANGUAGE"):
+                            line = line.replace('"','')
+                        match_language = re.match(r'LANGUAGE=(.*)$',line)
+                        if match_language:
+                            language = match_language.group(1)
         if len(lang) == 0:
             # fall back is 'en_US'
             lang = 'en_US.UTF-8'
         if len(language) == 0:
             # LANGUAGE has not been defined, generate a string from the provided LANG value
             language = self.makeEnvString(lang)
-        result.append(lang)
+        if len(formats) == 0:
+            formats = lang
+        result.append(formats)
         result.append(language)
         return result
 
-    def isCompleteSystemLanguage(self):
+    def isSetSystemFormats(self):
         if not os.access(self.environments[0], os.R_OK):
             return False
-        language_vars = {}
-        for line in open(self.environments[0]):
-            for var in 'LANGUAGE', 'LC_MESSAGES', 'LC_CTYPE', 'LC_COLLATE':
-                if line.startswith("%s=" % var):
-                    language_vars[var] = 1
-        if len(language_vars) < 4:
-            return False
-        return True
-
-    def isCompleteUserLanguage(self):
-        fname = os.path.expanduser('~/.profile')
-        if not os.access(fname, os.R_OK):
-            return False
-        language_vars = {}
-        for line in open(fname):
-            if not line.startswith('export'):
-                continue
-            for var in 'LANGUAGE', 'LC_MESSAGES', 'LC_CTYPE', 'LC_COLLATE':
-                if line.startswith('export %s=' % var):
-                    language_vars[var] = 1
-        if len(language_vars) < 4:
-            return False
-        return True
+        with open(self.environments[0]) as f:
+            for line in f:
+                if line.startswith("LC_TIME="):
+                    return True
+        return False
 
 
 if __name__ == "__main__":
     datadir = "/usr/share/language-selector/"
     li = LocaleInfo("languagelist", datadir)
 
-    print "default system locale and languages: '%s'" % li.getSystemDefaultLanguage()
-    print "default user locale and languages: '%s'" % li.getUserDefaultLanguage()
+    print("default system locale and languages: '%s'" % li.getSystemDefaultLanguage())
+    print("default user locale and languages: '%s'" % li.getUserDefaultLanguage())
 
-    print li._lang
-    print li._country
-    print li._languagelist
-    print li.generated_locales()
+    print(li._lang)
+    print(li._country)
+    print(li._languagelist)
+    print(li.generated_locales())
